<?php

/**
 * @file
 * Builds placeholder replacement tokens for webform_booking elements.
 */

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;

/**
 * Implements hook_token_info().
 *
 * Declares custom tokens for webform booking elements.
 */
function webform_booking_token_info() {
  $date_config_url = Url::fromRoute('entity.date_format.collection')->toString();

  $booking = [];
  $booking['booking'] = [
    'name' => t('Webform booking information'),
    'dynamic' => TRUE,
    'description' => Markup::create(t('Date, time or number of slots of a booking.') .
    _webform_token_render_more(t('Learn about booking tokens'),
    t("Webform Booking allows to display either the date, time or number of slots of a booking. The date can be formatted in any format defined in your site's <a href=\"@datetimeformats_link\">Date and Time formats</a> configuration.", [
      "@datetimeformats_link" => $date_config_url,
    ]) . '<br />' .
    t("To output individual elements, replace the '?' with …") . '<br />' .
    '<ul>' .
    '<li>
      <b>element_key</b><br />' .
      t("Default setting. Displays the raw date and time.") .
    '</li>' .
    '<li>
      <b>element_key:date</b><br />' .
      t("Displays the date formatted in the built in <i>html_date</i> format.") .
    '</li>' .
    '<li>
      <b>element_key:date:*</b><br />' .
      t("To select a date format, replace the '*' with the key of a <a href=\"@datetimeformats_link\">custom date and time format</a>.", [
        "@datetimeformats_link" => $date_config_url,
      ]) .
    '</li>' .
    '<li>
      <b>element_key:time</b><br />' .
      t("Displays the booking time formatted as 'H:i'.") .
    '</li>' .
    '<li>
      <b>element_key:slots</b><br />' .
      t("Displays the number of slots booked.") .
    '</li>' .
    '</ul>'))
    ];

  $tokens = [];
  $tokens['webform_submission'] = $booking;
  return ['types' => [], 'tokens' => $tokens];

}

/**
 * Implements hook_tokens().
 *
 * Generates tokens for date, time and slot information that can be used in
 * confirmation pages or emails generated by the Webform module.
 *
 * @param string $type
 *   The type of token. Should be 'webform_submission' in this implementation.
 * @param array $tokens
 *   The tokens to process.
 * @param array $data
 *   Data containing the webform submission object.
 * @param array $options
 *   Options array for token replacement.
 * @param \Drupal\Core\Render\BubbleableMetadata $bubbleable_metadata
 *   Metadata to propagate to the rendered output.
 *
 * @return array
 *   An array of token replacements.
 */
function webform_booking_tokens($type, $tokens, array $data, array $options, BubbleableMetadata $bubbleable_metadata) {

  $token_service = \Drupal::token();
  $replacements = [];

  if ($type === 'webform_submission' && !empty($data['webform_submission'])) {
    /** @var \Drupal\webform\WebformSubmissionInterface $webform_submission */
    $webform_submission = $data['webform_submission'];

    $value_tokens = $token_service->findWithPrefix($tokens, 'booking');
    if (empty($value_tokens)) {
      return $replacements;
    }

    foreach ($value_tokens as $value_token => $original) {
      $keys = explode(':', $value_token);
      // Extract name of booking form element.
      $element_key = array_shift($keys);
      // Extract element part to retrieve. Either 'slots' or 'date'.
      $element_part = array_shift($keys);
      // Get element format string. Default to built in html_date format.
      $element_format = !empty($keys[0]) ? array_shift($keys) : 'html_date';

      // Extract slot date, time and count, eg. "2025-02-17 11:40|1" and
      // perform basic validation of input string.
      $element_value = $webform_submission->getElementData($element_key);
      $pattern = '/^\d{4}-\d{2}-\d{2} \d{2}:\d{2}\|\d+$/';
      if (preg_match($pattern, $element_value) !== 1) {
        return $replacements;
      }
      $slot_data = explode('|', $element_value);
      $slot_datetime = new DrupalDateTime($slot_data[0]);
      if ($slot_datetime->hasErrors()) {
        return $replacements;
      }
      $slot_count = empty($slot_data[1]) ? 0 : $slot_data[1];

      switch ($element_part) {
        // Number of slots booked.
        case 'slots':
          $value = $slot_count ?? 0;
          break;

        // Booked time only.
        case 'time':
          $value = _webform_booking_format_date($slot_datetime, 'custom', 'H:i');
          break;

        // Booked date only.
        case 'date':
          $value = _webform_booking_format_date($slot_datetime, $element_format);
          break;

        // Raw value, ie. date and time.
        default:
          $value = $slot_data[0];

      };

      if ($value !== NULL) {
        $replacements[$original] = $value;
      }
    }

  }

  return $replacements;
}

/**
 * Formats the given date and time object.
 *
 * @param \Drupal\Core\Datetime\DrupalDateTime $dt
 *   The date and time object to format.
 * @param string $format
 *   Accepts a built-in format or a format defined in /admin/config/regional/date-time,
 *   or 'custom' for custom formatting.
 * @param string $datestring
 *   If $format is 'custom', this string is used as the format pattern.
 *
 * @return string
 *   The formatted date string.
 *
 * @throws \InvalidArgumentException
 *   Thrown if the provided date object is invalid or if the format is unsupported.
 */
function _webform_booking_format_date(\Drupal\Core\Datetime\DrupalDateTime $dt, string $format = 'short', string $datestring = '') {
  return \Drupal::service('date.formatter')->format(
    $dt->getTimestamp(),
    $format,
    $datestring
  );
}
